"""
Copyright (C) 2021 Clariteia SL

This file is part of minos framework.

Minos framework can not be copied and/or distributed without the express permission of Clariteia SL.
"""
__version__ = "0.0.4"

from .launchers import (
    EntrypointLauncher,
)
from .service import (
    ApiGatewayRestService,
)
